(******************************************************************************)
(*     Alt-Ergo: The SMT Solver For Software Verification                     *)
(*     Copyright (C) 2013-2015 --- OCamlPro                                   *)
(*     This file is distributed under the terms of the CeCILL-C licence       *)
(******************************************************************************)

(******************************************************************************)
(*     The Alt-Ergo theorem prover                                            *)
(*     Copyright (C) 2006-2013                                                *)
(*     CNRS - INRIA - Universite Paris Sud                                    *)
(*                                                                            *)
(*     Sylvain Conchon                                                        *)
(*     Evelyne Contejean                                                      *)
(*                                                                            *)
(*     Francois Bobot                                                         *)
(*     Mohamed Iguernelala                                                    *)
(*     Stephane Lescuyer                                                      *)
(*     Alain Mebsout                                                          *)
(*                                                                            *)
(*   This file is distributed under the terms of the CeCILL-C licence         *)
(******************************************************************************)

type gsubst = {
  sbs : Term.t Term.Subst.t;
  sty : Ty.subst;
  gen : int ;     (* l'age d'une substitution est l'age du plus vieux
		     terme qu'elle contient *)
  goal : bool;    (* vrai si la substitution contient un terme ayant un lien
		     avec le but de la PO *)
  s_term_orig : Term.t list;
}

module type EM_SIG = sig

  type t
  type xenv
  type form

  type trigger_info = {
    trigger : Formula.trigger;
    trigger_age : int ;  (* age d'un trigger *)
    trigger_orig : form ; (* lemme d'origine *)
    trigger_formula : form ; (* formule associee au trigger *)
    trigger_dep : Explanation.t ;
  }

  type term_info = {
    term_age : int ;        (* age du terme *)
    term_from_goal : bool ;   (* vrai si le terme provient du but de la PO *)
    term_from_formula : form option; (* lemme d'origine du terme *)
    term_from_terms : Term.t list;
  }

  type info = {
    age : int ; (* age du terme *)
    lem_orig : form list ; (* lemme d'ou provient eventuellement le terme *)
    t_orig : Term.t list;
    but : bool  (* le terme a-t-il un lien avec le but final de la PO *)
  }

  val empty : t

  val add_term : term_info -> Term.t -> t -> t
  val add_trigger : trigger_info -> t -> t
  val info : t -> info Term.Map.t
  val query : t -> xenv -> (trigger_info * gsubst list) list
end

module type TH_SIG = sig
  type t
  val are_equal : t -> Term.t -> Term.t -> Sig.answer
  val term_repr : t -> Term.t -> Term.t
  val class_of : t -> Term.t -> Term.t list
end

module EMatch (X : TH_SIG)(Form : sig type t end) : EM_SIG
  with type xenv = X.t and type form = Form.t
